@@ -674,7 +674,7 @@ def write_full_report(
 
     with open(str(output_report_txt).replace('.txt', '.html'), 'w') as outputfile:
         script_dir = Path(__file__).resolve().parent
-        html_template_path = script_dir / 'cropclassification' / 'postprocess' / 'html_rapport_template.html'
+        html_template_path = script_dir / 'html_rapport_template.html'
         html_template_file = open(html_template_path).read()                        
         src = Template(html_template_file)
         # replace strings and write to file
